/*
 * Copyright (c) 2005, Adam Gleave
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the 
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the copyright holders nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT OWNER AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef P_ARR_H
#define P_ARR_H

/* PURPOSE
 * Provides a safe value to set a p_arr structure to (safe = use before
 * initiliazation kills the program).
 * EXAMPLES
 * p_arr arri_owns = P_ARR_SAFE_DEF; */
 
#define P_ARR_SAFE_DEF { NULL, 0, 0, 0, 1, 1 }
 
/* PURPOSE
 * Provides a safe value to set a pointer to a p_arr structure to (safe =
 * use before initiliazation kills the program).
 * EXAMPLES
 * p_arr *arri_owns = P_ARR_POINTER_SAFE_DEF; */
 
#define P_ARR_POINTER_SAFE_DEF NULL

struct p_priv_arr {
  /* Pointer */
  
  /* We use char so that we can do pointer arithmetic in bytes. */
  /* array = pointer to array. */
  unsigned char       *array; 
  
  /* Buffer Information */
	
  /* alloced = amount of space allocated for buffer so far. */
  size_t               alloced;
  /* limit = maximium size buffer can grow to. */
  size_t               limit;
  /* used = amount of buffer that has been used. */
  size_t               used;

  /* Flags */

  /* hard = will failure kill program? hard == 1 == yes. */
  unsigned int         hard :1; 
  /* readonly = is buffer modifiable? readonly == 1 == not modifiable. */
  unsigned int         readonly :1;  
};

typedef struct p_priv_arr p_arr; 

#endif /* P_ARR_H */
