/*
 * Copyright (c) 2005, Adam Gleave
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the 
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the copyright holders nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT OWNER AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef P_BUFDANG_H
#define P_BUFDANG_H

#include "arr.h"
#include "returns.h"

/*
 * @WARNING: These functions are dangerous and should be considered bad 
 * things. Unfortunately, they're also indespensable in some cases.
 * Use with extreme caution.
 */ 

/* PURPOSE
 * To truncate p_buf to size p_size.
 * PARAMATERS
 * p_buf        - Pointer to p_arr structure.
 * p_size       - New size of p_buf.
 * RETURN VALUES
 * Returns P_RET_OK on success, P_RET_FAIL on failure.
 * WARNINGS
 * Truncating represents a possible loss of data (duh). */

int p_buf_truncate(p_arr *p_buf, size_t p_size);

/* PURPOSE
 * To grow the buffer by p_size.
 * PARAMATERS
 * p_buf        - Pointer to p_arr strucutre.
 * p_size       - Size increase of p_buf. 
 * RETURN VALUES
 * Returns P_RET_OK on success, P_RET_FAIL on failure.
 * WARNINGS
 * Although this isn't directly dangerous, why do you need to use it?
 * The only reason that I can think of you would need this is if you
 * are directly modifying the buffer. BAAD! */

int p_buf_grow(p_arr *p_buf, size_t p_size);

/* PURPOSE
 * To retrieve the buffer belonging to p_buf.
 * PARAMATERS
 * p_buf        - Pointer to p_arr structure.
 * RETURN VALUES
 * Returns p_buf's buffer on success, NULL on failure.
 * WARNINGS
 * You shouldn't be directly accessing the buffer - nuff said. */

void *p_buf_read_modifiable(p_arr *p_buf);

/* PURPOSE
 * To retrieve the buffer belonging to p_buf at position p_pos
 * checking that buffer is large enough to read p_size bytes.
 * PARAMATERS
 * p_buf        - Pointer to p_arr structure.
 * p_pos        - Position in p_buf to read from.
 * p_size       - Number of bytes to read.
 * RETURN VALUES
 * Returns p_buf's buffer with p_pos added to it on success, NULL on failure. 
 * WARNINGS
 * You shouldn't be directly accessing the buffer - nuff said. */

void *p_buf_read_pos_modifiable(p_arr *p_buf, size_t p_pos, size_t p_size);

#endif /* P_BUFDANG_H */
