/*
 * An application to print hello world.
 *
 * Copyright (c) 2005, Adam Gleave
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the 
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the copyright holders nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT OWNER AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWAR, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>

#include "buffer.h"
#include "io.h"

#define ROK P_RET_OK

void f_die(char *p_text)
{
  fprintf(stderr, "%s\n", p_text);
  exit(1);
}

int main(void)
{
  const char msg[] = "Hello World!\n";
  size_t msgsize = sizeof(msg);
  p_arr buf;
  
  /* NOTE: We don't error check because it will die on failure - you can
   * use the _soft option (i.e. p_buf_init_static_soft) to make it return
   * failure instead. */
  p_buf_init_static(&buf, msgsize);
  
  p_buf_write(&buf, 0, msg, msgsize);
  
  if (p_io_fwrite(stdout, &buf, &msgsize) != 0)
    f_die("main: p_io_fwrite stream failure.");
  
  if (msgsize < sizeof(msg))
    f_die("main: p_io_fwrite wrote less than we requested.");
  
  /* Finally, we end what could have been a two liner without Arri. */
  return 0;
}
