/*
 * Copyright (c) 2005, Adam Gleave
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the 
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the copyright holders nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT OWNER AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef P_IO_H
#define P_IO_H

#include "arr.h"

#define P_IO_RET_NOINTFAIL 0
#define P_IO_RET_INTFAIL 1

/* PURPOSE
 * To append *p_size bytes from p_stream to p_buf.
 * PARAMATERS
 * p_buf        - Pointer to p_arr structure.
 * p_stream     - STDIO stream.
 * p_size       - Pointer to number of bytes to read.
 * RETURN VALUES
 * 1 (P_IO_RET_INTFAIL) on non-stream failure, 0 (P_IO_RET_NOINTFAIL) on no
 * non-stream failure. *p_size is set to number of bytes read. *p_size is
 * smaller than its original value when end-of-file or a stream error occurs.
 * The standard library functions should be used to determine which occured. */

int p_io_fread(p_arr *p_buf, FILE *p_stream, size_t *p_size);

/* PURPOSE
 * To write *p_size bytes from p_buf into p_stream.
 * PARAMATERS
 * p_stream     - STDIO stream.
 * p_buf        - Pointer to p_arr structure.
 * p_size       - Pointer to number of bytes to write.
 * RETURN VALUES
 * 1 (P_IO_RET_INTFAIL) on non-stream failure, 0 (P_IO_RET_NOINTFAIL) on no
 * non-stream failure. *p_size is set to number of bytes written. *p_size is
 * smaller than its original value when end-of-file or a stream error occurs.
 * The standard library functions should be used to determine which occured. */

int p_io_fwrite(FILE *p_stream, p_arr *p_buf, size_t *p_size);

#endif /* P_IO_H */
