/*
 * Test of buffer.c. Part of Arri Buffer API's tests. 
 *
 * Copyright (c) 2005, Adam Gleave
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the 
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the copyright holders nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT OWNER AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * @WARNING: This code is intended to break Arri Buffer API. Because of this
 * objective this code does not represent valid or good use of Arri and,
 * in some cases, good programming practice.
 */

#include <stdio.h>
#include <string.h>

#include "buffer.h"
#include "bufdang.h"

#include "tests.h"

#define ROK P_RET_OK
#define RFAIL P_RET_FAIL

#define RSAME P_BUF_RET_SAME
#define RDIFF P_BUF_RET_DIFF

int main(void)
{
  p_arr *bufp;
  p_arr buf;
  const char *p;
  const char *p2;
  
  printf("Arri Buffer API test suite: buffer.c.\n\n");

  /* Initializaion test. */
  printf("Starting initialization test. ");

  p_buf_init_static(&buf, 1);
  p_buf_freebuf(&buf);

  if (p_buf_init_static_soft(&buf, 1) != ROK)
    LDIE("main: p_buf_init_static_soft: failed.");
  
  p_buf_freebuf(&buf);

  p_buf_init_dynamic(&buf, 1, 2);
  p_buf_freebuf(&buf);

  if (p_buf_init_dynamic_soft(&buf, 1, 2) != ROK)
    LDIE("main: p_buf_init_dynamic_soft: failed.");
  
  p_buf_freebuf(&buf);
  
  printf("PASSED.\n");
  
  /* Creation Test */
  printf("Starting creation test. ");

  bufp = p_buf_create_static(1);
  p_buf_free(bufp);

  if ((bufp = p_buf_create_static_soft(1)) == NULL)
    LDIE("main: p_buf_create_static_soft: failed.");
  
  p_buf_free(bufp);

  bufp = p_buf_create_dynamic(1, 2);
  p_buf_free(bufp);

  if ((bufp = p_buf_create_dynamic_soft(1, 2)) == NULL)
    LDIE("main: p_buf_create_dynamic_soft: failed.");
  
  p_buf_free(bufp);

  printf("PASSED.\n");
  /* Editing Test */
  printf("Starting writing test. ");

  if ((bufp = p_buf_create_dynamic_soft(5, 10)) == NULL)
    LDIE("main: p_buf_create_dynamic_soft: failed.");
  
  USEDCHECK(bufp, 0);
  
  if (p_buf_write(bufp, 0, "1234", 4) != ROK)
    LBUG("main: p_buf_write: won't write.");
  
  USEDCHECK(bufp, 4);

  if (p_buf_write(bufp, 4, "567891", 6) != ROK)
    LBUG("main: p_buf_write: won't write.");
  
  USEDCHECK(bufp, 10);
  
  /* 
   * @WARNING: INVALID WRITE.
   */
  if (p_buf_write(bufp, 12, "123456789", 9) != RFAIL)
    LBUG("main: p_buf_write: didn't fail.");
  
  USEDCHECK(bufp, 10);
  READCMP(bufp, "1234567891", 10);
  
  p_buf_empty(bufp);

  USEDCHECK(bufp, 0);
  
  if (p_buf_append_data(bufp, "1257", 4) != ROK)
    LBUG("main: p_buf_append_data: won't append.");
  
  USEDCHECK(bufp, 4);
  READCMP(bufp, "1257", 4);
  
  if (p_buf_insert_data(bufp, 2, "34", 2) != ROK)
    LBUG("main: p_buf_insert_data: won't insert.");
  
  USEDCHECK(bufp, 6);
  READCMP(bufp, "123457", 6);
  
  p_buf_init_static(&buf, 4);
  
  if (p_buf_write(&buf, 0, "6891", 4) != ROK)
    LBUG("main: p_buf_write: won't write.");
  
  if (p_buf_insert(bufp, 5, &buf, 0, 1) != ROK)
    LBUG("main: p_buf_insert: won't insert.");
  
  USEDCHECK(bufp, 7);
  READCMP(bufp, "1234567", 7);
  
  if (p_buf_copy(bufp, 7, &buf, 1, 3) != ROK)
    LBUG("main: p_buf_copy: won't copy.");

  USEDCHECK(bufp, 10);
  READCMP(bufp, "1234567891", 10);

  p_buf_empty(bufp);

  if (p_buf_append(bufp, &buf, 0, 4) != ROK)
    LBUG("main: p_buf_append: won't append.");
  
  USEDCHECK(bufp, 4);
  READCMP(bufp, "6891", 4);

  if (p_buf_append_buf(bufp, &buf) != ROK)
    LBUG("main: p_buf_append_buf: won't append.");
  
  USEDCHECK(bufp, 8);
  READCMP(bufp, "68916891", 8);

  if (p_buf_duplicate(bufp, &buf) != ROK)
    LBUG("main: p_buf_duplicate: won't duplicate.");
  
  USEDCHECK(bufp, 4);
  READCMP(bufp, "6891", 4);

  if (p_buf_delete(bufp, 0, 2) != ROK)
    LBUG("main: p_buf_delete: won't delete.");
  
  USEDCHECK(bufp, 2);
  READCMP(bufp, "91", 2);

  if (p_buf_delete(bufp, 0, 2) != ROK)
    LBUG("main: p_buf_delete: won't delete.");
  
  USEDCHECK(bufp, 0);

  USEDCHECK(&buf, 4);
  p_buf_empty(&buf);
  USEDCHECK(&buf, 0);

  p_buf_freebuf(&buf);
  p_buf_free(bufp);

  /* NOTE: We've already tested this, but do it anyway so that we keep
   * testing functions in sequential order. */

  printf("PASSED.\n");

  /* Editing test for dangerous functions */
  printf("Starting editing test for dangerous functions. ");
  
  if ((bufp = p_buf_create_dynamic_soft(5, 10)) == NULL)
    LDIE("main: p_buf_create_dynamic_soft: failed.");
  
  if (p_buf_truncate(bufp, 6) != RFAIL)
    LBUG("main: p_buf_truncate: didn't fail.");
  
  if (p_buf_append_data(bufp, "12", 2) != ROK)
    LBUG("main: p_buf_append_data: won't append.");
  
  if (p_buf_truncate(bufp, 1) != ROK)
    LBUG("main: p_buf_truncate: won't truncate.");
  
  USEDCHECK(bufp, 1);

  if (bufp->alloced != 5)
    LBUG("main: bufp->alloced != 5");
  
  if (p_buf_grow(bufp, 5) != ROK)
    LDIE("main: p_buf_grow: won't grow.");
  
  if (bufp->alloced != 6)
  {
    LBUG("main: bufp->alloced != 6");
  }
  
  if (p_buf_grow(bufp, 5) != RFAIL)
    LBUG("main: p_buf_grow: doesn't fail.");
  
  p_buf_free(bufp);
  
  printf("PASSED.\n");

  /* Reading test. */
  printf("Starting reading test. ");

  p_buf_init_static(&buf, 5);
  
  p_buf_append_data(&buf, "12345", 5);

  if ((p = p_buf_read(&buf)) == NULL)
    LBUG("main: p_buf_read: returned NULL.");
  
  if ((p2 = p_buf_read_pos(&buf, 0, 5)) == NULL)
    LBUG("main: p_buf_read_pos: returned NULL.");
  
  if (p != p2)
    LBUG("main: p != p2.");

  printf("PASSED.\n");
  
  /* Reading test for dangerous functions. */
  printf("Starting reading test for dangerous functions. ");
  
  if ((p = p_buf_read_modifiable(&buf)) == NULL)
    LBUG("main: p_buf_read_modifiable: returned NULL.");
  
  if (p != p2)
    LBUG("main: p != p2,");

  if ((p = p_buf_read_pos_modifiable(&buf, 0, 5)) == NULL)
    LBUG("main: p_buf_read_pos_modifiable: returned NULL.");
  
  if (p != p2)
    LBUG("main: p != p2.");
  
  p_buf_freebuf(&buf);
  
  printf("PASSED.\n");

  /* Test for option setters. */
  printf("Starting option setting test. ");
  
  if (p_buf_init_static_soft(&buf, 1) != ROK)
    LDIE("main: p_buf_init_static_soft: failed.");
  
  p_buf_append_data(&buf, "1", 1);

  p_buf_set_readonly(&buf);
  
  READCMP(&buf, "1", 1);

  if (p_buf_write(&buf, 0, "1", 1) != RFAIL)
    LBUG("main: p_buf_write: didn't fail.");
  
  READCMP(&buf, "1", 1);

  p_buf_freebuf(&buf);

  printf("PASSED.\n");
  
  /* Operation test. */
  printf("Starting operation test. ");

  p_buf_init_static(&buf, 2);
  bufp = p_buf_create_dynamic(1, 2);

  if (p_buf_cmp(bufp, &buf) != RSAME)
    LBUG("main: p_buf_cmp: aren't equal.");
  
  p_buf_append_data(&buf, "12", 2);

  if (p_buf_cmp(bufp, &buf) != RDIFF)
    LBUG("main: p_buf_cmp: aren't unequal.");
  
  p_buf_append_data(bufp, "2", 1);

  if (p_buf_cmp(bufp, &buf) != RDIFF)
    LBUG("main: p_buf_cmp: aren't unequal.");
  
  p_buf_insert_data(bufp, 0, "1", 1);

  if (p_buf_cmp(bufp, &buf) != RSAME)
    LBUG("main: p_buf_cmp: aren't equal.");

  printf("PASSED.\n");

  /* END OF TEST */
  printf("End of test for buffer.c: PASSED.\n");

  return 0;
}
