/*
 * Test of io.c. Part of Arri Buffer API's tests.  
 *
 * Copyright (c) 2005, Adam Gleave
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the 
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the copyright holders nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT OWNER AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * @WARNING: This code is intended to break Arri Buffer API. Because of this
 * objective this code does not represent valid or good use of Arri and,
 * in some cases, good programming practice.
 */

#include <stdio.h>

#include "buffer.h"
#include "io.h"

#include "tests.h"

#define ROK P_RET_OK

#define RSAME P_BUF_RET_SAME

#define RIONOINTFAIL P_IO_RET_NOINTFAIL

int main(void)
{
  p_arr *bufp1, *bufp2;
  size_t size;
  FILE *file;

  printf("Arri Buffer API test suite: io.c\n\n");

  printf("Starting test. ");

  /* Test. */

  if ((bufp1 = p_buf_create_static_soft(5)) == NULL)
    LDIE("main: p_buf_init_static_soft: failed.");
  
  if ((file = fopen("io-test-tmp-file.tmp", "w+")) == NULL)
    LDIE("main: fopen: failed.");

  if (p_buf_append_data(bufp1, "12345", 5) != ROK)
    LBUG("main: p_buf_append_data: won't append.");
  
  size = 5;
  
  if (p_io_fwrite(file, bufp1, &size) != RIONOINTFAIL)
    LBUG("main: p_io_fwrite: internal failure.");
  
  if (size < 5)
    LBUG("main: p_io_fwrite: wrote less than requested.");
  else if (size > 5)
    LBUG("main: p_io_fwrite: wrote more than requested.");
  
  if ((bufp2 = p_buf_create_static_soft(5)) == NULL)
    LDIE("main: p_buf_init_static_soft: failed.");
  
  rewind(file);
  
  if (p_io_fread(bufp2, file, &size) != RIONOINTFAIL)
    LBUG("main: p_io_fread: internal failure.");
  
  if (size < 5)
    LBUG("main: p_io_fread: read less than requested.");
  else if (size > 5)
    LBUG("main: p_io_fread: read more than requested.");
  
  if (p_buf_cmp(bufp1, bufp2) != RSAME)
    LBUG("main: p_buf_cmp: not equal.");
  
  p_buf_free(bufp1);
  p_buf_free(bufp2);

  printf("PASSED.\n");

  /* END OF TEST. */

  return 0;
}
  
  
  
