/*
 * Test of iounix.c. Part of Arri Buffer API's tests. 
 *
 * Copyright (c) 2005, Adam Gleave
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the 
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the copyright holders nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT OWNER AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * @WARNING: This code is intended to break Arri Buffer API. Because of this
 * objective this code does not represent valid or good use of Arri and,
 * in some cases, good programming practice.
 */

#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>

#include "buffer.h"
#include "iounix.h"

#include "tests.h"

#define ROK P_RET_OK

#define RSAME P_BUF_RET_SAME

#define RIOPINTFAIL P_IOP_RET_INTFAIL
#define RIOPEXTFAIL P_IOP_RET_EXTFAIL

#define RIOPPARTWROTE P_IOP_RET_PARTWROTE
#define RIOPWROTE P_IOP_RET_WROTE

#define RIOPPARTREAD P_IOP_RET_PARTREAD
#define RIOPREAD P_IOP_RET_READ

#define CHECKIOPRETCOM(p_func) \
  if (ret == RIOPINTFAIL) \
    LBUG("main: " p_func ": internal failure."); \
  else if (ret == RIOPEXTFAIL) \
    LBUG("main: " p_func ": external failure.")

int main(void)
{
  p_arr *bufp1, *bufp2;
  int fd;
  size_t size;
  ssize_t ret;
  
  printf("Arri Buffer API test suite: iounix.c\n\n");
  
  printf("Starting test. ");

  /* Test. */

  if ((bufp1 = p_buf_create_static_soft(5)) == NULL)
    LDIE("main: p_buf_create_static_soft: failed.");
  
  if ((fd = open("iounix-test-file-tmp.tmpfile", O_RDWR | O_CREAT | O_EXCL, 0600)) < 0)
    LDIE("main: open: won't open.");
  
  if (p_buf_append_data(bufp1, "12345", 5) != ROK)
    LBUG("main: p_buf_append_data: won't append.");
  
  ret = p_io_write(fd, bufp1, 5);
  
  CHECKIOPRETCOM("p_io_write");
  else if (ret < 5)
    LBUG("main: p_io_write: wrote less than requested.");
  else if (ret > 5)
    LBUG("main: p_io_write: wrote more than requested.");
  
  if ((bufp2 = p_buf_create_static_soft(5)) == NULL)
    LDIE("main: p_buf_create_static_soft: failed.");
  
  lseek(fd, 0, SEEK_SET); 
  
  ret = p_io_read(bufp2, fd, 5);

  CHECKIOPRETCOM("p_io_read");
  else if (ret < 5)
    LBUG("main: p_io_read: read less than requested.");
  else if (ret > 5)
    LBUG("main: p_io_read: read more than requested.");
  
  if (p_buf_cmp(bufp1, bufp2) != RSAME)
    LBUG("main: p_buf_cmp: not equal.");
  
  size = 5;
  ret = p_io_write_all(fd, bufp1, &size);
  CHECKIOPRETCOM("p_io_write_all");
  else if (ret == RIOPPARTWROTE)
    LBUG("main: p_io_write_all: partial write.");
  else if (ret != RIOPWROTE)
    LBUGPRINTF("main: p_io_write_all: unexpected return value %d.", ret);
  
  lseek(fd, 5, SEEK_SET);
  
  p_buf_empty(bufp2);
  size = 5;
  ret = p_io_read_all(bufp2, fd, &size);
  CHECKIOPRETCOM("p_io_read_all");
  else if (ret == RIOPPARTREAD)
    LBUG("main: p_io_read_all: partial read.");
  else if (ret != RIOPREAD)
    LBUGPRINTF("main: p_io_read_all: unexpected return value %d.", ret);
  
  p_buf_free(bufp1);
  p_buf_free(bufp2);

  printf("PASSED.\n");
  
  /* END OF TEST. */

  return 0;
}
