/*
 * Test of net.c. Part of Arri Buffer API's tests.
 *
 * Copyright (c) 2005, Adam Gleave
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the 
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the copyright holders nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT OWNER AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * @WARNING: This code is intended to break Arri Buffer API. Because of this
 * objective this code does not represent valid or good use of Arri and,
 * in some cases, good programming practice.
 */

#include <stdio.h>

#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/un.h>

#include "buffer.h"
#include "net.h"

#include "tests.h"

#define ROK P_RET_OK

#define RNETINTFAIL P_NET_RET_INTFAIL
#define RNETEXTFAIL P_NET_RET_EXTFAIL

#define RNETPARTSENT P_NET_RET_PARTSENT
#define RNETSENT P_NET_RET_SENT

#define CHECKNETRETCOM(p_func) \
  if (ret == RNETINTFAIL) \
    LBUG("main: " p_func ": internal failure."); \
  else if (ret == RNETEXTFAIL) \
    LBUG("main: " p_func ": external failure.")

int main(void)
{
  p_arr *bufp1, *bufp2;
  int s[2];
  ssize_t ret;
  size_t size;
  const void *p; 

  printf("Arri Buffer API test suite: net.c\n\n");

  /* Test. */
  printf("Starting test. ");

  if (socketpair(AF_UNIX, SOCK_STREAM, 0, s) == -1)
    LDIE("main: socketpair: failed.");
  
  if ((bufp1 = p_buf_create_static(113)) == NULL)
    LDIE("main: p_buf_create_static: failed.");
  
  if (p_buf_append_data(bufp1, "AAAAAcQBQ4HDAUKCQ1AWU4ORBsAGhcAFgAcChRIHh0"
                        "FAg4NSR8WDEkHF0keFQ4HBFUdHEgVCgMWRUYFBwIACQ4JThcOV"
                        "S8KHFIHEQIGCk8FG0lz", 110) != ROK)
    LBUG("main: p_buf_append_data: failed.");
  
  ret = p_net_sendto(s[1], bufp1, NULL, 0, 0, 1); 
  
  CHECKNETRETCOM("p_net_sendto");
  else if (ret == 0)
    LBUG("main: p_net_sendto: sent nothing.");
  else if (ret != 1)
    LBUGPRINTF("main: p_net_sendto: unexpected return value %d.", ret);
  
  if ((bufp2 = p_buf_create_static(113)) == NULL)
    LDIE("main: p_buf_create_static: failed.");
  
  ret = p_net_recvfrom(bufp2, s[0], NULL, 0, 0, 1);

  CHECKNETRETCOM("p_net_recvfrom");
  else if (ret == 0)
    LBUG("main: p_net_recvfrom: received nothing.");
  else if (ret != 1)
    LBUGPRINTF("main: p_net_recvfrom: unexpected return value %d.", ret);
  
  USEDCHECK(bufp2, 1);
  READCMP(bufp2, "A", 1);
  
  size = 1;
  ret = p_net_sendto_all(s[1], bufp1, NULL, 0, 0, &size);

  CHECKNETRETCOM("p_net_sendto_all");
  else if (ret == RNETPARTSENT)
    LBUGPRINTF("main: p_net_sendto_all: partial send - size is %d.", size);
  else if (ret != RNETSENT)
    LBUGPRINTF("main: p_net_sendto_all: unexpected return value %d. "
               "size is %d.", size);
  else {
    if (size != 1)
      LBUGPRINTF("main: p_net_sendto_all: unexpected size value %d.", size);
  }    
    
  
  ret = p_net_recv(bufp2, s[0], 0, 1);
  
  CHECKNETRETCOM("p_net_recv");
  else if (ret == 0)
    LBUG("main: p_net_recv: received nothing.");
  else if (ret != 1)
    LBUGPRINTF("main: p_net_recv: unexpected return value %d.", ret);
  
  USEDCHECK(bufp2, 2);
  READCMP(bufp2, "AA", 2);

  ret = p_net_send(s[1], bufp1, 0, 1);

  CHECKNETRETCOM("p_net_send");
  else if (ret == 0)
    LBUG("main: p_net_send: sent nothing.");
  else if (ret != 1)
    LBUGPRINTF("main: p_net_sendto: unexpected return value %d.", ret);
  
  ret = p_net_recv(bufp2, s[0], 0, 1);

  CHECKNETRETCOM("p_net_recv");
  else if (ret == 0)
    LBUG("main: p_net_recv: received nothing.");
  else if (ret != 1)
    LBUGPRINTF("main: p_net_recv: unexpected return value %d.", ret);
  
  USEDCHECK(bufp2, 3);
  READCMP(bufp2, "AAA", 3);
  
  size = 110;
  ret = p_net_send_all(s[1], bufp1, 0, &size);

  CHECKNETRETCOM("p_net_send_all");
  else if (ret == RNETPARTSENT)
    LBUGPRINTF("main: p_net_send_all: partial send - size is %d.", size);
  else if (ret != RNETSENT)
    LBUGPRINTF("main: p_net_send_all: unexpected return value %d. "
               "size is %d.", size);
  else {
    if (size != 110)
      LBUGPRINTF("main: p_net_send_all: unexpected size value %d.", size);
  }
  
  ret = p_net_recv(bufp2, s[0], MSG_WAITALL, 110);
  
  CHECKNETRETCOM("p_net_recv");
  else if (ret == 0)
    LBUG("main: p_net_recv: reveived nothing.");
  else if (ret != 110)
    LBUGPRINTF("main: p_net_recv: unexpected return value %d.", ret);
  
  /* Correct bufp1 to be what should be recieved. */
  p_buf_insert_data(bufp1, 0, "AAA", 3);
  
  USEDCHECK(bufp2, 113);
  if (p_buf_cmp(bufp1, bufp2) != P_BUF_RET_SAME)
    LBUG("main: p_buf_cmp: not equal.");
  
  p_buf_free(bufp1);
  p_buf_free(bufp2);
  
  printf("PASSED.\n");

  /* END OF TEST. */
  
  return 0;
}
